package nz.ac.massey.programming_project_159333_s1_2021;

import java.util.BitSet;

/**
 * Hello world!
 */
public final class App {
    private App() {
    }

    /**
     * Says hello to the world.
     * @param args The arguments of the program.
     * @throws InterruptedException
     */
    public static void main(String[] args) throws InterruptedException {
        ECS gameEngine = new ECS();
        System.out.println("Hello World!");
        gameEngine.registerComponent(Vec2D.class.getName());
        gameEngine.registerComponent(RidgidBody.class.getName());
        gameEngine.registerComponent(Gravity.class.getName());

        PhysicsSystem physicsSystem = new PhysicsSystem(gameEngine);
        gameEngine.registerSystem(PhysicsSystem.class.getName(), physicsSystem);
        {
            BitSet signature = new BitSet();
            signature.set(gameEngine.getComponentIndex(Gravity.class.getName()));
            signature.set(gameEngine.getComponentIndex(Vec2D.class.getName()));
            signature.set(gameEngine.getComponentIndex(RidgidBody.class.getName()));
            gameEngine.setSystemSignature(PhysicsSystem.class.getName(), signature);
        }

        physicsSystem.init();

        LogVec2DSystem logVec2DSystem = new LogVec2DSystem(gameEngine);
        gameEngine.registerSystem(LogVec2DSystem.class.getName(), logVec2DSystem);
        {
            BitSet signature = new BitSet();
            signature.set(gameEngine.getComponentIndex(Gravity.class.getName()));
            signature.set(gameEngine.getComponentIndex(Vec2D.class.getName()));
            signature.set(gameEngine.getComponentIndex(RidgidBody.class.getName()));
            gameEngine.setSystemSignature(LogVec2DSystem.class.getName(), signature);
        }

        logVec2DSystem.init();

        FrameRateSystem frameRateSystem = new FrameRateSystem();
        gameEngine.registerSystem(FrameRateSystem.class.getName(), frameRateSystem);

        // Create components:

        int entity = gameEngine.createEntity();

        gameEngine.addComponent(entity, Gravity.class.getName(), new Gravity(0.0, -9.80665, -1.0, 100.0));
        gameEngine.addComponent(entity, Vec2D.class.getName(), new Vec2D(1.5, 0.0, 1));
        gameEngine.addComponent(entity, RidgidBody.class.getName(), new RidgidBody());


        
        double dt = 0.0;

        double frameRate = 1.0 / 20; // 1 second divided by target number of frames
        double idleTime = 0.0; // the amount of time spent sleeping

        boolean quit = false;
        long loopStart = System.currentTimeMillis();
        while (!quit){
            // Get the time between frames for accurate time-based calculations (i.e. physics systems etc)
            long startTime = System.nanoTime();

            frameRateSystem.update(dt, idleTime);
            physicsSystem.update(dt);
            logVec2DSystem.update(dt);
            dt = (System.nanoTime() - startTime) / 1e9; // convert nanoseconds to seconds

            // Limit frame rate (approximately) to minimum of 16 ms
            if (dt < frameRate){
                idleTime = frameRate-dt;
                Thread.sleep((int)((frameRate-dt)*1000));
                dt = (System.nanoTime() - startTime) / 1e9;
            }
            if (System.currentTimeMillis() - loopStart >= 1000){
                loopStart = System.currentTimeMillis();
                System.out.println("Second Elapsed");
            }
        }
    }
}
