package nz.ac.massey.programming_project_159333_s1_2021;
/**
 * System Manager
 * This class manages systems registrations, and keeps a current list
 * of all entities the system should operate on
 * 
 * Contributors:
 * Brychan Dempsey - brychand@hotmail.com
 * 
 */

import java.util.BitSet;
import java.util.Map;
import java.util.HashMap;

class SystemManager{
    Map<String, BitSet> signatures = new HashMap<>();
    Map<String, ECSSystem> systems = new HashMap<>();

    // Registering the system adds it to the array of systems.
    // In Austin Morlan's implementation, this also creates an instance of the
    // system that can be called from the main thread.
    // It returns this object.
    // In Java, we need to initialise the object first (before this is called),
    // then register that object. Enactment of the system is performed on that 
    // instance.
    // I.e., create an object that represents the system class; then store that class in the system
    // table.
    public boolean registerSystem(String systemName, ECSSystem system){
        if (systems.containsKey(systemName)){
            System.err.println("System already registered");
            return false;
        }
        systems.put(systemName, system);
        signatures.put(systemName, new BitSet());
        return true;
    }

    public void setSignature(String system, BitSet registrations){
        signatures.put(system, registrations);
    }

    public void entityDestroyed(int entity){
        for (String key : systems.keySet()) {
            systems.get(key).entities.remove(entity);
        }
    }

    public void entitySignatureChanged(int entity, BitSet entitySignature){
        for (String key : systems.keySet()) {
            // Check if the signature is null
            if (!entitySignature.equals(null)){
                BitSet srcCpy = (BitSet)entitySignature.clone();
                srcCpy.and(signatures.get(key));
                if (srcCpy.equals(signatures.get(key))){ // Bitwise check if the entity is subscribed to this system
                    systems.get(key).entities.add(entity);
                }
                else{
                    systems.get(key).entities.remove(entity);
                }
            }
        }
    }
}